package com.jenny.magic.items;

import com.mojang.datafixers.util.Pair;
import net.minecraft.ChatFormatting;
import net.minecraft.core.BlockPos;
import net.minecraft.core.Holder;
import net.minecraft.network.chat.Component;
import net.minecraft.server.level.ServerLevel;
import net.minecraft.world.InteractionHand;
import net.minecraft.world.InteractionResult;
import net.minecraft.world.InteractionResultHolder;
import net.minecraft.world.entity.player.Player;
import net.minecraft.world.item.ItemStack;
import net.minecraft.world.item.context.UseOnContext;
import net.minecraft.world.level.Level;
import net.minecraft.world.level.biome.Biome;
import org.jetbrains.annotations.NotNull;

import java.util.Objects;
import java.util.function.Predicate;

import static com.jenny.magic.Magic.MODID;

public class TeleportScrollBiome extends BaseItem {
    private final Biome biome;
    private final ChatFormatting color;

    public TeleportScrollBiome(Properties pProperties, Biome biome, ChatFormatting color) {
        super(pProperties);
        this.biome = biome;
        this.color = color;
    }

    @Override
    public @NotNull InteractionResultHolder<ItemStack> use(@NotNull Level pLevel, @NotNull Player pPlayer, @NotNull InteractionHand pUsedHand) {
        use(pPlayer, pPlayer.getItemInHand(pUsedHand));
        return InteractionResultHolder.success(pPlayer.getItemInHand(pUsedHand));
    }

    @Override
    public @NotNull InteractionResult useOn(@NotNull UseOnContext pContext) {
        use(pContext.getPlayer(), pContext.getItemInHand());
        return InteractionResult.SUCCESS;
    }

    protected void use(@NotNull Player player, ItemStack itemStack) {
        if (!player.level().isClientSide) {
            Predicate<Holder<Biome>> predicate = new Predicate<Holder<Biome>>() {
                @Override
                public boolean test(@NotNull Holder<Biome> biomeHolder) {
                    return Objects.equals(biomeHolder.get().toString(), biome.toString());
                }
            };
            Pair<BlockPos, Holder<Biome>> pair = ((ServerLevel) player.level()).findClosestBiome3d(predicate, player.blockPosition(), 69000, 32, 64);
            if (pair == null) {
                MessageBiomeNotFound(player);
            } else {
                player.teleportTo(pair.getFirst().getX(), pair.getFirst().getY(), pair.getFirst().getZ());
            }
        }
        itemStack.shrink(1);
    }

    protected void MessageBiomeNotFound(Player player) {
        player.sendSystemMessage(Component.translatable(String.format("message.%s.scroll_teleport_biome.not_found", MODID)));
    }
}
