package com.jenny.magic.items;

import com.mojang.datafixers.util.Pair;
import net.minecraft.core.BlockPos;
import net.minecraft.core.Holder;
import net.minecraft.nbt.CompoundTag;
import net.minecraft.resources.ResourceLocation;
import net.minecraft.server.level.ServerLevel;
import net.minecraft.world.InteractionHand;
import net.minecraft.world.InteractionResult;
import net.minecraft.world.InteractionResultHolder;
import net.minecraft.world.entity.player.Player;
import net.minecraft.world.item.ItemStack;
import net.minecraft.world.item.context.UseOnContext;
import net.minecraft.world.level.Level;
import net.minecraft.world.level.biome.Biome;
import net.minecraftforge.registries.ForgeRegistries;
import org.jetbrains.annotations.NotNull;

import java.util.Collection;
import java.util.Objects;
import java.util.function.Predicate;

public class TeleportScrollRandomBiome extends BaseItem {
    public TeleportScrollRandomBiome(Properties pProperties) {
        super(pProperties);
    }

    @Override
    public @NotNull InteractionResultHolder<ItemStack> use(@NotNull Level pLevel, @NotNull Player pPlayer, @NotNull InteractionHand pUsedHand) {
        use(pPlayer, pPlayer.getItemInHand(pUsedHand));
        return InteractionResultHolder.success(pPlayer.getItemInHand(pUsedHand));
    }

    @Override
    public @NotNull InteractionResult useOn(@NotNull UseOnContext pContext) {
        use(pContext.getPlayer(), pContext.getItemInHand());
        return InteractionResult.SUCCESS;
    }

    protected void use(@NotNull Player player, ItemStack itemStack) {
        if (!player.level().isClientSide && itemStack.getTag() != null && itemStack.getTag().contains("teleport_biome")) {
            Biome biome = ForgeRegistries.BIOMES.getValue(ResourceLocation.parse(itemStack.getTag().getString("teleport_biome")));
            Predicate<Holder<Biome>> predicate = new Predicate<Holder<Biome>>() {
                @Override
                public boolean test(@NotNull Holder<Biome> biomeHolder) {
                    return Objects.equals(biomeHolder.get().toString(), biome.toString());
                }
            };
            Pair<BlockPos, Holder<Biome>> pair = ((ServerLevel) player.level()).findClosestBiome3d(predicate, player.blockPosition(), 69000, 32, 64);
            if (pair == null) {
                System.out.println("error: biome not found");
            } else {
                player.teleportTo(pair.getFirst().getX(), pair.getFirst().getY(), pair.getFirst().getZ());
            }
        } else {
            setBiome(player.level(), itemStack);
        }
    }

    protected void setBiome(Level level, ItemStack itemStack) {
        //ForgeRegistries.BIOMES.getEntries().stream().toList().get().;
        Collection<Biome> list = ForgeRegistries.BIOMES.getValues();
        //String biomeStr = list.get(level.getRandom().nextInt(list.size())).toString();

        CompoundTag compoundTag = itemStack.getTag();
        compoundTag = compoundTag != null ? compoundTag : new CompoundTag();
        //compoundTag.putString("teleport_biome", biomeStr);
        itemStack.setTag(compoundTag);
    }
}
